@@ -1,5 +1,5 @@
 # - perl *-
-# $Id: cssh.pl 220 2008-01-23 22:18:33Z duncan_ferguson $
+# $Id: cssh.pl 228 2008-02-26 20:46:35Z duncan_ferguson $
 #
 # Script:
 #   $RCSfile$
@@ -50,7 +50,7 @@
 #     http://www.opensource.org/licenses/gpl-license.php
 #
 ############################################################################
-my $VERSION = '$Revision: 220 $ ($Date: 2008-01-23 22:18:33 +0000 (Wed, 23 Jan 2008) $)';
+my $VERSION = '$Revision: 228 $ ($Date: 2008-02-26 20:46:35 +0000 (Tue, 26 Feb 2008) $)';
 
 # Now tidy it up, but in such as way cvs doesn't kill the tidy up stuff
 $VERSION =~ s/\$Revision: //;
@@ -85,7 +85,7 @@ use Net::hostent;
 my $scriptname = $0;
 $scriptname =~ s!.*/!!;    # get the script name, minus the path
 
-my $options = 'dDv?hHuqQgGist:T:c:l:o:e:';    # Command line options list
+my $options = 'dDv?hHuqQgGist:T:c:l:o:e:C:';    # Command line options list
 my %options;
 my %config;
 my $debug = 0;
@@ -231,8 +231,7 @@ sub parse_config_file($) {
     s/\s*$//;                       # remove trailing whitespace
     chomp();
 
-    #my ($key, $value) = split(/[ 	]*=[ 	]*/);
-    /(\w+)[   ]*=[  ]*(.*)/;
+    next unless m/\s*(\S+)\s*=\s*(.*)\s*/;
     my ( $key, $value ) = ( $1, $2 );
     $config{$key} = $value;
     logmsg( 3, "$key=$value" );
@@ -345,6 +344,9 @@ sub check_config() {
 sub load_configfile() {
   parse_config_file( $sysconfigdir . '/csshrc' );
   parse_config_file( $ENV{HOME} . '/.csshrc' );
+  if ( $options{C} && -r $options{C} ) {
+    parse_config_file( $options{C} );
+  }
   check_config();
 }
 
@@ -371,7 +373,7 @@ sub check_ssh_hostnames {
 
   if ( -r $ssh_config && open( SSHCFG, "<", $ssh_config ) ) {
     while (<SSHCFG>) {
-      next unless (m/^\s*host\s+([\w\.-]+)/);
+      next unless (m/^\s*host\s+([\w\.-]+)/i);
       $ssh_hostnames{$1} = 1;
     }
     close(SSHCFG);
@@ -823,6 +825,10 @@ sub setup_helper_script() {
 
 sub check_host($) {
   my $host = shift;
+  if ( $host =~ m/^(\d{1,3}\.?){4}$/ ) {
+    logmsg( 2, "Not resolving IP address '$host'" );
+    return 1;
+  }
   if ( $config{method} eq "ssh" ) {
     logmsg( 1, "Attempting name resolution via user ssh config file" );
     if ( $ssh_hostnames{$host} ) {
@@ -1323,7 +1329,7 @@ sub add_host_by_name() {
 sub build_hosts_menu() {
   logmsg( 2, "Building hosts menu" );
 
-  # first, emtpy the hosts menu from the 4th entry on
+  # first, empty the hosts menu from the 4th entry on
   my $menu = $menus{bar}->entrycget( 'Hosts', -menu );
   $menu->delete( 6, 'end' );
 
@@ -1407,6 +1413,23 @@ sub setup_repeat() {
   logmsg( 2, "Repeat setup" );
 }
 
+sub write_default_user_config() {
+  return if ( !$ENV{HOME} || -e "$ENV{HOME}/.csshrc" );
+
+  if ( open( CONFIG, ">", "$ENV{HOME}/.csshrc" ) ) {
+    foreach ( sort( keys(%config) ) ) {
+
+      # do not output internal vars
+      next if ( $_ =~ /^internal/ );
+      print CONFIG "$_=$config{$_}\n";
+    }
+    close(CONFIG);
+  }
+  else {
+    logmsg( 1, "Unable to write default $ENV{HOME}/.csshrc file" );
+  }
+}
+
 ### Window and menu definitions ###
 
 sub create_windows() {
@@ -1518,7 +1541,7 @@ sub create_windows() {
   $windows{addhost} = $windows{main_window}->DialogBox(
     -popover        => $windows{main_window},
     -popanchor      => 'n',
-    -title          => "Add Host",
+    -title          => "Add Host(s) or Cluster(s)",
     -buttons        => [ 'Add', 'Cancel' ],
     -default_button => 'Add',
   );
@@ -1719,11 +1742,12 @@ sub create_menubar() {
     -menuitems => [
       [
         "command",
-        "Retile Hosts",
+        "Retile Windows",
         -command     => \&retile_hosts,
         -accelerator => $config{key_retilehosts},
       ],
-      [ "command", "Capture Terminal",    -command => \&capture_terminal, ],
+
+#         [ "command", "Capture Terminal",    -command => \&capture_terminal, ],
       [ "command", "Toggle active state", -command => \&toggle_active_state, ],
       [
         "command",
@@ -1732,7 +1756,7 @@ sub create_menubar() {
       ],
       [
         "command",
-        "Add Host",
+        "Add Host(s) or Cluster(s)",
         -command     => \&add_host_by_name,
         -accelerator => $config{key_addhost},
       ],
@@ -1859,6 +1883,9 @@ $windows{main_window}->positionfrom('user')
 logmsg( 2, "Setting up repeat" );
 setup_repeat();
 
+logmsg( 2, "Writing default user configuration" );
+write_default_user_config();
+
 # Start event loop
 logmsg( 2, "Starting MainLoop" );
 MainLoop();
@@ -1933,14 +1960,6 @@ communications protocol instead of ssh.
 
 =item *
 
-Starting the utility will be much faster with a configuration file (as this
-prevents searching for required files).  Generate one containing all default
-entries with:
-
-C<< cssh -u > $HOME/.csshrc >>
-
-=item *
-
 When using cssh on a large number of systems to connect back to a single
 system (e.g. you issue a command to the cluster to scp a file from a given
 location) and when these connections require authentication (i.e. you are
@@ -2001,7 +2020,8 @@ Enable|Disable automatically quiting after the last client window has closed
 
 =item -u
 
-Output configuration in the format used by the F<$HOME/.csshrc> file
+Output the current configuration in the same format used by the 
+F<$HOME/.csshrc> file.
 
 =item -g|-G 
 
@@ -2011,6 +2031,10 @@ Enable|Disable window tiling (overriding the config file)
 
 Use supplied file as additional cluster file (see also L<"FILES">)
 
+=item -C <file>
+
+Use supplied file as additional configuration file (see also L<"FILES">)
+
 =item -l $LOGNAME
 
 Specify the default username to use for connections (if different from the
@@ -2083,7 +2107,8 @@ Quit the program and close all connections and windows
 
 =item Control-+
 
-Open the Add Host dialogue box
+Open the 'Add Host(s) or Cluster(s)' dialogue box.  Mutiple host or cluster names 
+can be entered, separated by spaces.
 
 =item Alt-n
 
@@ -2317,7 +2342,7 @@ left and then up
 
 B<NOTE:> The key shortcut modifiers must be in the form "Control", "Alt", or 
 "Shift", i.e. with the first letter capitalised and the rest lower case.  Keys
-may also be disabled individually by setting to the work "null".
+may also be disabled individually by setting to the word "null".
 
 =back
 
